@@ -2,6 +2,8 @@ from .container import Container
 from .page import Page
 from .utils import decode_text
 
+import pathlib
+import itertools
 from pdfminer.pdfparser import PDFParser
 from pdfminer.pdfdocument import PDFDocument
 from pdfminer.pdfpage import PDFPage
@@ -10,22 +12,17 @@ from pdfminer.layout import LAParams
 from pdfminer.converter import PDFPageAggregator
 from pdfminer.psparser import PSLiteral
 
-class PDF(Container):
-    cached_properties = Container.cached_properties + [ "_pages" ]
 
-    def __init__(self,
-        stream,
-        pages = None,
-        laparams = None,
-        precision = 0.001,
-        password = ""
-    ):
-        self.laparams = None if laparams == None else LAParams(**laparams)
+class PDF(Container):
+    cached_properties = Container.cached_properties + ["_pages"]
+
+    def __init__(self, stream, pages=None, laparams=None, precision=0.001, password=""):
+        self.laparams = None if laparams is None else LAParams(**laparams)
         self.stream = stream
         self.pages_to_parse = pages
         self.precision = precision
         rsrcmgr = PDFResourceManager()
-        self.doc = PDFDocument(PDFParser(stream), password = password)
+        self.doc = PDFDocument(PDFParser(stream), password=password)
         self.metadata = {}
         for info in self.doc.info:
             self.metadata.update(info)
@@ -44,8 +41,11 @@ class PDF(Container):
         self.interpreter = PDFPageInterpreter(rsrcmgr, self.device)
 
     @classmethod
-    def open(cls, path, **kwargs):
-        return cls(open(path, "rb"), **kwargs)
+    def open(cls, path_or_fp, **kwargs):
+        if isinstance(path_or_fp, (str, pathlib.Path)):
+            return cls(open(path_or_fp, "rb"), **kwargs)
+        else:
+            return cls(path_or_fp, **kwargs)
 
     def process_page(self, page):
         self.interpreter.process_page(page)
@@ -53,14 +53,16 @@ class PDF(Container):
 
     @property
     def pages(self):
-        if hasattr(self, "_pages"): return self._pages
+        if hasattr(self, "_pages"):
+            return self._pages
 
         doctop = 0
         pp = self.pages_to_parse
         self._pages = []
         for i, page in enumerate(PDFPage.create_pages(self.doc)):
-            page_number = i+1
-            if pp != None and page_number not in pp: continue
+            page_number = i + 1
+            if pp is not None and page_number not in pp:
+                continue
             p = Page(self, page, page_number=page_number, initial_doctop=doctop)
             self._pages.append(p)
             doctop += p.height
@@ -78,10 +80,21 @@ class PDF(Container):
 
     @property
     def objects(self):
-        if hasattr(self, "_objects"): return self._objects
+        if hasattr(self, "_objects"):
+            return self._objects
         all_objects = {}
         for p in self.pages:
             for kind in p.objects.keys():
                 all_objects[kind] = all_objects.get(kind, []) + p.objects[kind]
         self._objects = all_objects
         return self._objects
+
+    @property
+    def annots(self):
+        gen = (p.annots for p in self.pages)
+        return list(itertools.chain(*gen))
+
+    @property
+    def hyperlinks(self):
+        gen = (p.hyperlinks for p in self.pages)
+        return list(itertools.chain(*gen))
